
function setVideo(%gapi, %vendor, %renderer, %platform, %os, %arch)
{
   %profileVersion = 4;

   if (%vendor $= "unknown" || %renderer $= "unknown")
   {
      echo("Unknown vendor or renderer...not saving profile info yet");
      return;
   }

   echo("Checking video card profiles...");
   if ($pref::Video::profileVersion == %profileVersion &&
       $pref::Video::profiledVendor $= %vendor &&
       $pref::Video::profiledRenderer $= %renderer)
   {
      echo("...up to date.");
      return;
   }
   echo("Updating video card profile.");

   // set defaults
   $pref::Interior::lockArrays = true;
   $pref::TS::fogTexture = false;
   $pref::TS::skipFirstFog = false;
   $pref::OpenGL::disableEXTCompiledVertexArray = false;
   $pref::OpenGL::disableSubImage = false;
   $pref::OpenGL::noEnvColor = false;
   $pref::OpenGL::disableARBTextureCompression = false;
   $pref::OpenGL::noDrawArraysAlpha = false;
   $pref::OpenGL::maxTextureDim = 512;
   $pref::Video::only16 = false;
   $pref::Video::SafeModeOn = true;
   $pref::Video::clipHigh = false;
   $pref::Video::deleteContext = true;
   $pref::Video::simResolution = "800 600 32";
   $pref::Video::detailLevel = 2;

   if (%vendor $= "NVIDIA" || strstr(%renderer,"GeForce")!=-1 || strstr(%renderer,"RIVA")!=-1)
      setNVidia(%gapi, %vendor, %renderer, %platform, %os, %arch);
   else if (getSubStr(%vendor,0,3) $= "ATI" || strstr(%renderer,"Rage")!=-1 || strstr(%renderer,"RAGE")!=-1 || strstr(%renderer,"Radeon")!=-1)
      setATI(%gapi, %vendor, %renderer, %platform, %os, %arch);
   else if (%vendor $= "Matrox" || %renderer $= "G200" || %renderer $= "G400" || %renderer $= "G450")
      setMatrox(%gapi, %vendor, %renderer, %platform, %os, %arch);
   else if (%vendor $= "3Dfx" || %renderer $= "Banshee" || strstr(%renderer,"Voodoo")!=-1)
      setVoodoo(%gapi, %vendor, %renderer, %platform, %os, %arch);
   else if (%renderer $= "Savage2000")
      setViper(%gapi, %vendor, %renderer, %platform, %os, %arch);
   else if (%renderer $= "GLINT R3 PT" || %renderer $= "PowerVR KYRO")
      setPermedia(%gapi, %vendor, %renderer, %platform, %os, %arch);

   setThinkTanksDetailLevel($pref::Video::detailLevel);
   $pref::Video::profileVersion = %profileVersion;
   $pref::Video::profiledVendor = %vendor;
   $pref::Video::profiledRenderer = %renderer;
}

function setNVidia(%gapi, %vendor, %renderer, %platform, %os, %arch)
{
   if (strstr(%renderer,"TNT2")!=-1 || strstr(%renderer,"TNT 2")!=-1)
   {
      $pref::Video::detailLevel = 1;
      $pref::Video::simResolution = "640 480 32";
      $pref::OpenGL::maxTextureDim = 256;
   }
   else if (strstr(%renderer,"TNT")!=-1)
   {
      $pref::Video::detailLevel = 0;
      $pref::Video::simResolution = "640 480 16";
      $pref::OpenGL::maxTextureDim = 256;
   }
   else if (strstr(%renderer,"GeForce ")!=-1)
   {
      // GF1, because of space...
      $pref::Video::detail = 1;
   }
   else if (strstr(%renderer,"RIVA")!=-1 || strstr(%renderer,"Riva")!=-1)
   {
      $pref::Interior::lockArrays = false;
      $pref::Video::detailLevel = 0;
      $pref::Video::simResolution = "640 480 16";
      $pref::OpenGL::maxTextureDim = 256;
   }
}

function setATI(%gapi, %vendor, %renderer, %platform, %os, %arch)
{
   // start with standard ATI settings (Radeon on Windows)
   $pref::OpenGL::disableARBTextureCompression   = true;

   if ( (strstr(%renderer,"Rage 128") != -1) || (strstr(%renderer,"RAGE 128") != -1) || (strstr(%renderer,"ATI 3D RAGE") != -1))
   {
      $pref::TS::fogTexture = true;
      $pref::OpenGL::noEnvColor = true;
      $pref::OpenGL::disableARBTextureCompression   = false;
      $pref::Video::detailLevel = 0;
      $pref::Video::simResolution = "640 480 16";
      $pref::OpenGL::maxTextureDim = 256;
      if (%platform $= "linux")
         $pref::OpenGL::DisableEXTCompiledVertexArray = 1;

   }
   if (%renderer $= "RAGE 128 Pro Dual")
      $pref::Video::only16 = true;
   if (%renderer $= "Radeon DDR x86/MMX/3DNow!")
      $pref::Interior::lockArrays = false;

   if (strstr(%renderer,"Radeon") != -1)
   {
      $pref::OpenGL::DisableEXTCompiledVertexArray = 1;
      $pref::OpenGL::DisableEXTFogCoord = 1;
      if (strstr(%renderer,"Radeon 70") != -1)
      {
         $pref::Video::simResolution = "640 480 16";
         $pref::Video::detailLevel = 1;
      }
      else if (strstr(%renderer,"Radeon 7") != -1)
      {
         $pref::Video::simResolution = "640 480 32";
         $pref::Video::detailLevel = 1;
      }
      else
      {
         // else 800 x 600, hi
         $pref::OpenGL::DisableEXTCompiledVertexArray = 0;
         $pref::OpenGL::DisableEXTFogCoord = 0;
      }
   }

   if (%platform $= "mac")
   {
      // mac ati cards don't seem to need these disabled
      $pref::OpenGL::DisableEXTCompiledVertexArray = 0;
      $pref::OpenGL::DisableEXTFogCoord = 0;
   }
}

function setMatrox(%gapi, %vendor, %renderer, %platform, %os, %arch)
{
   $pref::OpenGL::disableSubImage = true;
   $pref::OpenGL::noEnvColor = true;
   $pref::Video::detailLevel = 1;
   if (%platform $= "linux")
      $pref::OpenGL::DisableEXTCompiledVertexArray = 1;
}

function setVoodoo(%gapi, %vendor, %renderer, %platform, %os, %arch)
{
   // first apply banshee settings
   $pref::TS::fogTexture = true;
   $pref::OpenGL::noEnvColor = true;
   $pref::Video::deleteContext = false;
   $pref::TS::skipFirstFog = true;
   $pref::OpenGL::noDrawArraysAlpha = true;
   $pref::OpenGL::maxTextureDim = 256;

   $pref::Video::simResolution = "640 480 16";

   if (%renderer $= "Voodoo3")
   {
      $pref::OpenGL::disableEXTCompiledVertexArray = true;
      $pref::Video::deleteContext = true;
      $pref::Video::detailLevel = 0;
      $pref::Video::simResolution = "640 480 16";
   }
   else if (%renderer $= "Voodoo4" || %renderer $= "Voodoo5")
   {
//      $pref::OpenGL::disableEXTCompiledVertexArray = true;
      $pref::TS::fogTexture = false;
      $pref::TS::skipFirstFog = false;
      $pref::OpenGL::noDrawArraysAlpha = false;
      $pref::Video::detailLevel = 1;
      $pref::Video::simResolution = "800 600 16";
   }

   if (%os $= "W2K" && (%renderer $= "Voodoo3" || %renderer $= "Voodoo5"))
      // whacky, whacky, world
      $pref::Video::deleteContext = false;
}

function setPermedia(%gapi, %vendor, %renderer, %platform, %os, %arch)
{
   $pref::Video::detailLevel = 0;
   $pref::Video::simResolution = "640 480 16";
   $pref::OpenGL::maxTextureDim = 256;
}

function setViper(%gapi, %vendor, %renderer, %platform, %os, %arch)
{
   $pref::OpenGL::disableEXTCompiledVertexArray = true;
   $pref::OpenGL::noEnvColor = true;
   $pref::Video::detailLevel = 1;
   $pref::Video::simResolution = "640 480 16";
}
